@@ -1,6 +1,7 @@
 
 import concurrent.futures
 import json
+import os
 import time
 
 from typing import Union, List
@@ -38,6 +39,8 @@ from assemblyline.odm.models.workflow import Workflow
 
 config = forge.get_config()
 
+THREAD_POOL_SIZE = int(os.environ.get("POOL_SIZE", 20))
+
 
 class AssemblylineDatastore(object):
     def __init__(self, datastore_object):
@@ -530,6 +533,7 @@ class AssemblylineDatastore(object):
     @elasticapm.capture_span(span_type='datastore')
     def get_or_create_file_tree(self, submission, max_depth, cl_engine=forge.get_classification(),
                                 user_classification=None):
+        # Generate cache key
         if user_classification is not None:
             user_classification = cl_engine.normalize_classification(user_classification, long_format=False)
             cache_key = f"{submission['sid']}_{user_classification}_supp"
@@ -538,16 +542,20 @@ class AssemblylineDatastore(object):
         else:
             cache_key = f"{submission['sid']}_supp"
 
+        # Transform submission into primitives
         if isinstance(submission, Model):
             submission = submission.as_primitives()
 
+        # Get number of files and score
         num_files = len(list(set([x[:64] for x in submission['results']])))
         max_score = submission['max_score']
 
+        # Load / Validate cache tree if exist
         cached_tree = self.submission_tree.get_if_exists(cache_key, as_obj=False)
         if cached_tree:
             tree = json.loads(cached_tree['tree'])
             if self._is_valid_tree(tree, num_files, max_score):
+                log.debug(f"File tree for submission '{submission['sid']}' was loaded from cache.")
                 return {
                     "tree": tree,
                     "classification": cached_tree['classification'],
@@ -556,11 +564,19 @@ class AssemblylineDatastore(object):
                     "supplementary": json.loads(cached_tree['supplementary'])
                 }
 
-        partial = submission['state'] != 'completed'
+        # Set default state
+        file_data_map = {}
         files = {}
+        forbidden_files = set()
+        max_classification = cl_engine.UNRESTRICTED
+        missing_files = {}
+        names = {}
+        partial = submission['state'] != 'completed'
         scores = {}
-        missing_files = []
         supplementary = []
+        tree_cache = set()
+
+        # Get file data
         file_hashes = [x[:64] for x in submission['results']]
         file_hashes.extend([x[:64] for x in submission['errors']])
         file_hashes.extend([f['sha256'] for f in submission['files']])
@@ -569,12 +585,9 @@ class AssemblylineDatastore(object):
         except MultiKeyError as e:
             log.warning(f"Trying to generate file tree but we are missing file(s): {str(e.keys)}")
             temp_file_data_map = e.partial_output
-            missing_files = e.keys
+            missing_files = set(e.keys)
             partial = True
-        forbidden_files = set()
 
-        max_classification = cl_engine.UNRESTRICTED
-        file_data_map = {}
         for key, value in temp_file_data_map.items():
             if user_classification and not cl_engine.is_accessible(user_classification, value['classification']):
                 partial = True
@@ -583,6 +596,7 @@ class AssemblylineDatastore(object):
             file_data_map[key] = value
             max_classification = cl_engine.max_classification(max_classification, value['classification'])
 
+        # Get result data
         try:
             results_data = self.result.multiget([x for x in submission['results'] if not x.endswith(".e")],
                                                 as_obj=False)
@@ -591,6 +605,7 @@ class AssemblylineDatastore(object):
             results_data = e.partial_output
             partial = True
 
+        # Scan for extracted/supplementary files and their names
         for key, item in results_data.items():
             sha256 = key[:64]
 
@@ -601,106 +616,78 @@ class AssemblylineDatastore(object):
 
             # Get files
             extracted = item['response']['extracted']
-            if len(extracted) == 0:
-                continue
-            if sha256 not in files:
-                files[sha256] = []
+
+            # Get file names
+            for e in extracted:
+                names.setdefault(e['sha256'], set())
+                names[e['sha256']].add(e['name'])
+
+            files.setdefault(sha256, [])
             files[sha256].extend(extracted)
 
             # Get supplementary files
-            if len(item['response']['supplementary']) == 0:
-                continue
             supplementary.extend([x['sha256'] for x in item['response']['supplementary']])
 
-        tree_cache = []
-
-        def recurse_tree(child_p, placeholder, parents_p, lvl=0):
-            if lvl == max_depth + 1:
-                # Enforce depth protection while building the tree
+        # Process a file and its children
+        def process_file(current_file, tree_branch, partial, lvl=0):
+            # Enforce depth protection while building the tree
+            if lvl >= max_depth + 1:
                 return
 
-            c_sha256 = child_p['sha256']
-            c_name = child_p['name']
-            if c_sha256 in placeholder:
-                placeholder[c_sha256]['name'].append(c_name)
-            else:
-                children_list = {}
+            # Get information about the file
+            file_sha256 = current_file['sha256']
+            file_name = current_file['name']
+
+            # Check if the file not already in the tree and if its allowed to be processed
+            if file_sha256 not in tree_branch \
+                    and file_sha256 not in forbidden_files \
+                    and file_sha256 not in missing_files:
+
+                # Set default state for the file
+                children = {}
                 truncated = False
-                child_list = files.get(c_sha256, [])
-                for new_child in child_list:
+
+                # Load file data information if still missing
+                if file_sha256 not in file_data_map:
+                    file_data_map[file_sha256] = self.file.get(file_sha256, as_obj=False)
+
+                # If file data still can't be found, bail out
+                if file_sha256 not in file_data_map:
+                    log.warning(f"Trying to generate file tree but we are missing a file: {file_sha256}")
+                    partial = True
+                    return
+
+                # Process each children of the file
+                for new_child in files.get(file_sha256, []):
+                    # Check if the file has already been processed elsewhere in the tree
                     if new_child['sha256'] in tree_cache:
                         truncated = True
-                        continue
-                    tree_cache.append(child['sha256'])
+                    else:
+                        # Process file children
+                        tree_cache.add(new_child['sha256'])
+                        process_file(new_child, children, partial, lvl + 1)
 
-                    if new_child['sha256'] not in parents_p:
-                        recurse_tree(new_child, children_list,
-                                     parents_p + [c_sha256], lvl + 1)
-
-                try:
-                    placeholder[c_sha256] = {
-                        "name": [c_name],
-                        "type": file_data_map[c_sha256]['type'],
-                        "sha256": file_data_map[c_sha256]['sha256'],
-                        "size": file_data_map[c_sha256]['size'],
-                        "children": children_list,
-                        "truncated": truncated,
-                        "score": scores.get(c_sha256, 0),
-                    }
-                except KeyError:
-                    if c_sha256 not in forbidden_files and c_sha256 not in missing_files:
-                        file_data_map[c_sha256] = self.file.get(c_sha256, as_obj=False)
-                        placeholder[c_sha256] = {
-                            "name": [c_name],
-                            "type": file_data_map[c_sha256]['type'],
-                            "sha256": file_data_map[c_sha256]['sha256'],
-                            "size": file_data_map[c_sha256]['size'],
-                            "children": children_list,
-                            "truncated": truncated,
-                            "score": scores.get(c_sha256, 0),
-                        }
+                # Add file and it's children to the tree
+                tree_branch[file_sha256] = {
+                    "name": list(names.get(file_sha256, [file_name])),
+                    "type": file_data_map[file_sha256]['type'],
+                    "sha256": file_data_map[file_sha256]['sha256'],
+                    "size": file_data_map[file_sha256]['size'],
+                    "children": children,
+                    "truncated": truncated,
+                    "score": scores.get(file_sha256, 0),
+                }
 
+        # Build tree
         tree = {}
         for f in submission['files']:
-            sha256 = f['sha256']
-            name = f['name']
-
-            if sha256 in tree:
-                tree[sha256]['name'].append(name)
-            else:
-                parents = [sha256]
-                children = {}
-                c_list = files.get(sha256, [])
-                for child in c_list:
-                    tree_cache.append(child['sha256'])
-                    recurse_tree(child, children, parents)
-
-                try:
-                    tree[sha256] = {
-                        "name": [name],
-                        "children": children,
-                        "type": file_data_map[sha256]['type'],
-                        "sha256": file_data_map[sha256]['sha256'],
-                        "size": file_data_map[sha256]['size'],
-                        "truncated": False,
-                        "score": scores.get(sha256, 0),
-                    }
-                except KeyError:
-                    if sha256 not in forbidden_files and sha256 not in missing_files:
-                        file_data_map[sha256] = self.file.get(sha256, as_obj=False)
-                        tree[sha256] = {
-                            "name": [name],
-                            "children": children,
-                            "type": file_data_map[sha256]['type'],
-                            "sha256": file_data_map[sha256]['sha256'],
-                            "size": file_data_map[sha256]['size'],
-                            "truncated": False,
-                            "score": scores.get(sha256, 0),
-                        }
+            tree_cache.add(f['sha256'])
+            process_file(f, tree, partial)
 
         # Cleanup supplementary
         supplementary = list(set(supplementary))
 
+        # Create a cache entry for the tree if it's not partial
         if not partial:
             cached_tree = {
                 'expiry_ts': now_as_iso(config.datastore.ilm.days_until_archive * 24 * 60 * 60),
@@ -735,9 +722,12 @@ class AssemblylineDatastore(object):
         files = list(set(files))
 
         if len(files) < num_files:
+            log.warning(f"Invalid cached tree, number of files is not the same: {len(files)} != {num_files}")
             return False
 
         if tree_score != max_score:
+            log.warning(
+                f"Invalid cached tree, the tree score does not match the submission score: {tree_score} != {max_score}")
             return False
 
         return True
@@ -971,11 +961,12 @@ class AssemblylineDatastore(object):
 
     @elasticapm.capture_span(span_type='datastore')
     def get_stat_for_heuristic(self, p_id):
+        log.info(f"Generating stats for heuristic: {p_id})")
         query = f"result.sections.heuristic.heur_id:{p_id}"
         stats = self.ds.result.stats("result.score", query=query)
 
         if stats['count'] == 0:
-            up_stats = {'count': 0, 'min': 0, 'max': 0, 'avg': 0, 'sum': 0}
+            up_stats = {'count': 0, 'min': 0, 'max': 0, 'avg': 0, 'sum': 0, 'first_hit': None, 'last_hit': None}
         else:
             first = self.ds.result.search(query=query, fl='created', rows=1,
                                           sort="created asc", as_obj=False, use_archive=True)['items'][0]['created']
@@ -991,28 +982,46 @@ class AssemblylineDatastore(object):
                 'last_hit': last
             }
 
-        self.ds.heuristic.update(p_id, [
-            (self.ds.heuristic.UPDATE_SET, 'stats', up_stats)
-        ])
+            self.ds.heuristic.update(p_id, [
+                (self.ds.heuristic.UPDATE_SET, 'stats', up_stats)
+            ])
 
         return up_stats
 
     @elasticapm.capture_span(span_type='datastore')
     def calculate_heuristic_stats(self):
-        heur_list = sorted([(x['heur_id'])
-                            for x in self.ds.heuristic.stream_search("heur_id:*", fl="heur_id,name,classification",
-                                                                     as_obj=False)])
+        # Calculate stats for all heuristics
+        heuristics = [x['heur_id'] for x in self.ds.heuristic.stream_search("heur_id:*", fl="heur_id", as_obj=False)]
 
-        with concurrent.futures.ThreadPoolExecutor(max(min(len(heur_list), 20), 1)) as executor:
-            for heur_id in heur_list:
+        log.info(f"All {len(heuristics)} heuristics will have their statistics updated.")
+
+        # Update all heuristics found
+        with concurrent.futures.ThreadPoolExecutor(max(min(len(heuristics), THREAD_POOL_SIZE), 1)) as executor:
+            for heur_id in heuristics:
                 executor.submit(self.get_stat_for_heuristic, heur_id)
 
     @elasticapm.capture_span(span_type='datastore')
     def get_stat_for_signature(self, p_id, p_source, p_name, p_type):
+        if p_id is None:
+            log.info(f"Finding ID for {p_type.upper()} signature: \"{p_name}\" [{p_source}]")
+            try:
+                res = self.signature.search(f"type:\"{p_type}\" AND source:\"{p_source}\" AND name:\"{p_name}\"",
+                                            fl="id", as_obj=False)['items']
+                for item in res:
+                    p_id = item['id']
+            except Exception:
+                pass
+
+        if not p_id:
+            log.error(f"Failed to find ID for {p_type.upper()} signature: \"{p_name}\" [{p_source}]")
+            return None
+
+        log.info(f"Generating stats for {p_type.upper()} signature: {p_id}")
+
         query = f'result.sections.tags.file.rule.{p_type}:"{p_source}.{p_name}"'
         stats = self.ds.result.stats("result.score", query=query)
         if stats['count'] == 0:
-            up_stats = {'count': 0, 'min': 0, 'max': 0, 'avg': 0, 'sum': 0}
+            up_stats = {'count': 0, 'min': 0, 'max': 0, 'avg': 0, 'sum': 0, 'first_hit': None, 'last_hit': None}
         else:
             first = self.ds.result.search(query=query, fl='created', rows=1,
                                           sort="created asc", as_obj=False, use_archive=True)['items'][0]['created']
@@ -1028,22 +1037,45 @@ class AssemblylineDatastore(object):
                 'last_hit': last
             }
 
-        self.ds.signature.update(p_id, [
-            (self.ds.signature.UPDATE_SET, 'stats', up_stats)
-        ])
+            self.ds.signature.update(p_id, [
+                (self.ds.signature.UPDATE_SET, 'stats', up_stats)
+            ])
 
         return up_stats
 
     @elasticapm.capture_span(span_type='datastore')
-    def calculate_signature_stats(self):
-        sig_list = sorted([(x['id'], x['source'], x['name'], x['type'])
-                           for x in self.ds.signature.stream_search("id:*",
-                                                                    fl="id,name,type,source,classification",
-                                                                    as_obj=False)])
+    def calculate_signature_stats(self, lookback_time="now-1d"):
+        # Compute updated signatures since lookback time
+        signatures = set()
+        query = f"result.sections.tags.file.rule.\\*:* AND created:{{{lookback_time} TO now]"
+        fl = "created,result.sections.tags.file.rule.*"
 
-        with concurrent.futures.ThreadPoolExecutor(max(min(len(sig_list), 20), 1)) as executor:
-            for sid, source, name, sig_type in sig_list:
-                executor.submit(self.get_stat_for_signature, sid, source, name, sig_type)
+        new_time = None
+        for res in self.ds.result.stream_search(query, fl=fl, as_obj=False):
+            for sec in res['result']['sections']:
+                for rule_type, rules in sec['tags']['file']['rule'].items():
+                    for rule in rules:
+                        try:
+                            source, name = rule.split('.', 1)
+                            signatures.add((source, name, rule_type))
+                        except Exception:
+                            log.warning(f'Failed to parse rule name for rule: {rule} [{rule_type}]')
+
+            if new_time is None or res['created'] > new_time:
+                new_time = res['created']
+
+        log.info(f"{len(signatures)} signatures where triggered since: {lookback_time}.")
+
+        # Bail out if no signatures
+        if not signatures:
+            return lookback_time
+
+        # Update all signatures found
+        with concurrent.futures.ThreadPoolExecutor(max(min(len(signatures), THREAD_POOL_SIZE), 1)) as executor:
+            for source, name, sig_type in signatures:
+                executor.submit(self.get_stat_for_signature, None, source, name, sig_type)
+
+        return new_time
 
     @elasticapm.capture_span(span_type='datastore')
     def list_all_services(self, as_obj=True, full=False) -> Union[List[dict], List[Service]]:
