@@ -18,8 +18,9 @@ import logging
 import re
 import sys
 from datetime import datetime
-from typing import Any, Dict, Mapping, Tuple, Union
+from typing import Any as _Any, Dict, Mapping as _Mapping, Tuple, Union
 import arrow
+from assemblyline.common.net import is_valid_domain, is_valid_ip
 from dateutil.tz import tzutc
 
 from assemblyline.common import forge
@@ -45,7 +46,7 @@ UTC_TZ = tzutc()
 DOMAIN_REGEX = r"(?:(?:[A-Za-z0-9\u00a1-\uffff][A-Za-z0-9\u00a1-\uffff_-]{0,62})?[A-Za-z0-9\u00a1-\uffff]\.)+" \
                r"(?:xn--)?(?:[A-Za-z0-9\u00a1-\uffff]{2,}\.?)"
 DOMAIN_ONLY_REGEX = f"^{DOMAIN_REGEX}$"
-EMAIL_REGEX = f"^[a-zA-Z0-9!#$%&'*+/=?^_‘{{|}}~-]+(?:\\.[a-zA-Z0-9!#$%&'*+/=?^_‘{{|}}~-]+)*@{DOMAIN_REGEX}$"
+EMAIL_REGEX = f"^[a-zA-Z0-9!#$%&'*+/=?^_‘{{|}}~-]+(?:\\.[a-zA-Z0-9!#$%&'*+/=?^_‘{{|}}~-]+)*@({DOMAIN_REGEX})$"
 IP_REGEX = r"(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)"
 IP_ONLY_REGEX = f"^{IP_REGEX}$"
 PRIVATE_IP = r"(?:(?:127|10)(?:\.(?:[2](?:[0-5][0-5]|[01234][6-9])|[1][0-9][0-9]|[1-9][0-9]|[0-9])){3})|" \
@@ -58,15 +59,15 @@ SHA1_REGEX = r"^[a-f0-9]{40}$"
 SHA256_REGEX = r"^[a-f0-9]{64}$"
 MAC_REGEX = r"^(?:(?:[0-9a-f]{2}-){5}[0-9a-f]{2}|(?:[0-9a-f]{2}:){5}[0-9a-f]{2})$"
 URI_PATH = r"(?:[/?#]\S*)"
-FULL_URI = f"^((?:(?:[A-Za-z]*:)?//)?(?:\\S+(?::\\S*)?@)?(?:{IP_REGEX}|{DOMAIN_REGEX})(?::\\d{{2,5}})?){URI_PATH}?$"
+FULL_URI = f"^((?:(?:[A-Za-z]*:)?//)?(?:\\S+(?::\\S*)?@)?({IP_REGEX}|{DOMAIN_REGEX})(?::\\d{{2,5}})?){URI_PATH}?$"
 PLATFORM_REGEX = r"^(Windows|Linux|MacOS|Android|iOS)$"
 PROCESSOR_REGEX = r"^x(64|86)$"
 
 logger = logging.getLogger('assemblyline.odm')
 
 
-def flat_to_nested(data: dict[str, Any]) -> dict[str, Any]:
-    sub_data: dict[str, Any] = {}
+def flat_to_nested(data: dict[str, _Any]) -> dict[str, _Any]:
+    sub_data: dict[str, _Any] = {}
     nested_keys = []
     for key, value in data.items():
         if '.' in key:
@@ -351,6 +352,9 @@ class Domain(Keyword):
             raise ValueError(f"[{self.name or self.parent_name}] '{value}' not match the "
                              f"validator: {self.validation_regex.pattern}")
 
+        if not is_valid_domain(value):
+            raise ValueError(f"[{self.name or self.parent_name}] '{value}' has a non-valid TLD.")
+
         return value.lower()
 
 
@@ -363,10 +367,15 @@ class Email(Keyword):
         if not value:
             return None
 
-        if not self.validation_regex.match(value):
+        match = self.validation_regex.match(value)
+        if not match:
             raise ValueError(f"[{self.name or self.parent_name}] '{value}' not match the "
                              f"validator: {self.validation_regex.pattern}")
 
+        if not is_valid_domain(match.group(1)):
+            raise ValueError(f"[{self.name or self.parent_name}] '{match.group(1)}' in email '{value}'"
+                             " is not a valid Domain.")
+
         return value.lower()
 
 
@@ -384,6 +393,10 @@ class URI(Keyword):
             raise ValueError(f"[{self.name or self.parent_name}] '{value}' not match the "
                              f"validator: {self.validation_regex.pattern}")
 
+        if not is_valid_domain(match.group(2)) and not is_valid_ip(match.group(2)):
+            raise ValueError(f"[{self.name or self.parent_name}] '{match.group(2)}' in URI '{value}'"
+                             " is not a valid Domain or IP.")
+
         return match.group(0).replace(match.group(1), match.group(1).lower())
 
 
@@ -808,7 +821,7 @@ class Optional(_Field):
 
 class Model:
     @classmethod
-    def fields(cls, skip_mappings=False) -> Mapping[str, _Field]:
+    def fields(cls, skip_mappings=False) -> _Mapping[str, _Field]:
         """
         Describe the elements of the model.
 
@@ -862,7 +875,7 @@ class Model:
         return out
 
     @classmethod
-    def flat_fields(cls, show_compound=False, skip_mappings=False) -> Mapping[str, _Field]:
+    def flat_fields(cls, show_compound=False, skip_mappings=False) -> _Mapping[str, _Field]:
         """
         Describe the elements of the model.
 
@@ -883,7 +896,8 @@ class Model:
 
     @classmethod
     def markdown(cls, toc_depth=1, defaults=None) -> Union[str, Dict]:
-        markdown_content = "[comment]: # (AUTOGENERATED MARKDOWN CONTENT. UPDATES TO ODM DOCUMENTATION SHOULD BE DONE THROUGH ASSEMBLYLINE-BASE REPO!)\n"
+        markdown_content = "[comment]: # (AUTOGENERATED MARKDOWN CONTENT. UPDATES TO ODM DOCUMENTATION " \
+            "SHOULD BE DONE THROUGH ASSEMBLYLINE-BASE REPO!)\n"
 
         # Header
         markdown_content += f"{'#'*toc_depth} {cls.__name__}\n> {cls.__description}\n\n"
@@ -938,7 +952,10 @@ class Model:
                 description = f'{description}<br>Values:<br>`{", ".join(values)}`'
 
             # Is this a required field?
-            required = ":material-checkbox-marked-outline: Yes" if info.__class__ != Optional else ":material-minus-box-outline: Optional"
+            if info.__class__ != Optional:
+                required = ":material-checkbox-marked-outline: Yes"
+            else:
+                required = ":material-minus-box-outline: Optional"
 
             # Determine the correct default values to display
             default = f"`{info.default}`"
@@ -1162,7 +1179,7 @@ def _construct_field(field, value):
             return None, value
 
 
-def construct_safe(mod, data) -> tuple[Any, dict]:
+def construct_safe(mod, data) -> tuple[_Any, dict]:
     if not isinstance(data, dict):
         return None, data
     fields = mod.fields()
