@@ -86,6 +86,7 @@ class Tagging(odm.Model):
     @odm.model(index=True, store=False, description="General Information Tag Model")
     class Info(odm.Model):
         phone_number = odm.Optional(odm.List(odm.PhoneNumber(copyto="__text__")))
+        password = odm.Optional(odm.List(odm.Keyword(copyto="__text__")), description="Password")
 
     @odm.model(index=True, store=False, description="File Tag Model")
     class File(odm.Model):
@@ -358,6 +359,10 @@ class Tagging(odm.Model):
         class FileShortcut(odm.Model):
             command_line = odm.Optional(odm.List(odm.Keyword(copyto="__text__")), description="Command Line")
             icon_location = odm.Optional(odm.List(odm.Keyword(copyto="__text__")), description="Icon Location")
+            machine_id = odm.Optional(odm.List(odm.Keyword(copyto="__text__")), description="Machine ID")
+            tracker_mac = odm.Optional(
+                odm.List(odm.Keyword(copyto="__text__")), description="Possible MAC address from the Tracker block"
+            )
 
         @odm.model(index=True, store=False, description="Strings File Model")
         class FileStrings(odm.Model):
